@@ -1,6 +1,6 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2015,2016,2017,2018,2019,2020
-;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
+;;  Copyright (C) 2015-2025
+;;      "Mu Lei" known as "NalaGinrut" <mulei@gnu.org>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
 ;;  Lesser General Public License published by the Free Software
@@ -68,7 +68,7 @@
 
 (define (refresh-current-app)
   (let ((recompile (format #f "find ~a -name \"*.scm\" -exec touch {} \\;" (current-toplevel)))
-        (clean (format #f "rm -fr ~a/tmp/cache/tpl/*" (current-toplevel))))
+        (clean (format #f "rm -fr ~a/cache/tpl/*" (current-tmp))))
     (system recompile)
     (system clean)
     (run-hook *refresh-hook*)))
@@ -91,7 +91,7 @@
 (define *component-meta-table*
   `((controller . ,*controllers-table*)))
 (define (load-compent-rules component)
-  (define rf (string-append (current-toplevel) "/.route"))
+  (define rf (current-route-cache))
   (define (-> k)
     (module-ref (resolve-module '(artanis artanis)) k))
   (when (not (file-exists? rf))
@@ -113,17 +113,21 @@
 
 (define (clean-stuffs)
   (define toplevel (current-toplevel))
-  (define route-cache (format #f "~a/tmp/cache/route.cache" toplevel))
-  (define route (format #f "~a/.route" toplevel))
+  (define route-cache (format #f "~a/cache/route.cache" (current-tmp)))
+  (define route (format #f "~a/.route" (current-tmp)))
   (define-syntax-rule (clean-it f)
     (when (file-exists? f) (delete-file f)))
   (clean-it route-cache)
   (clean-it route))
 
+(define (load-plugins)
+  (load (format #f "~a/conf/plugins.scm" (current-toplevel))))
+
 (define (init-work)
   (clean-stuffs)
   (add-to-load-path (current-toplevel))
   (add-to-load-path (string-append (current-toplevel) "/lib"))
+  (load-plugins)
   (try-load-app)
   (register-rules)
   (load-rules))
