@@ -1,6 +1,6 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013-2020,2024
-;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
+;;  Copyright (C) 2013-2025
+;;      "Mu Lei" known as "NalaGinrut" <mulei@gnu.org>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
 ;;  Lesser General Public License published by the Free Software
@@ -34,6 +34,8 @@
   #:use-module (ice-9 receive)
   #:use-module (ice-9 q)
   #:use-module (ice-9 control)
+  #:use-module (ice-9 threads)
+  #:use-module (ice-9 control)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-19)
   #:use-module (web http)
@@ -99,7 +101,9 @@
             is-guile-compatible-server-core? positive-integer? negative-integer?
             io-exception:peer-is-shutdown? io-exception:out-of-memory?
             out-of-system-resources? allow-long-live-connection?
-            free-JS-announcement gen-cache-file current-route-cache)
+            free-JS-announcement gen-cache-file current-route-cache
+            gnu-locale->http-lang-tag http-lang-tag->gnu-locale
+            sxml-query sxml-replace!)
   #:re-export (the-environment
                utf8->string
                bytevector?
@@ -294,7 +298,10 @@
   ;; TODO: support AF_INET6 in the future
   (if (port-filename (request-port req))
       ;; Valid socket port
-      (inet-ntop AF_INET (sockaddr:addr (getpeername (request-port req))))
+      (catch #t
+        (lambda ()
+          (inet-ntop AF_INET (sockaddr:addr (getpeername (request-port req)))))
+        (lambda _ "Unknown IP"))
       "localtest")) ; fake hostname for testing
 
 (define-syntax-rule (remote-info req)
@@ -1082,23 +1089,26 @@
   (if (not (provided? 'posix))
       (lambda (thunk handler-thunk) (thunk))
       (lambda (thunk handler-thunk)
-        (let ((handler #f))
+        (let ((handler (make-parameter #f)))
           (catch 'interrupt
             (lambda ()
               (dynamic-wind
                   (lambda ()
-                    (set! handler
-                          (sigaction SIGINT (lambda (sig)
-                                              (run-when-sigint-hook)
-                                              (throw 'interrupt)))))
+                    (handler (sigaction SIGINT))
+                    (sigaction SIGINT (lambda (sigtmp)
+                                        (run-when-sigint-hook)
+                                        (throw 'interrupt))))
                   thunk
                   (lambda ()
-                    (if handler
+                    (if (handler)
                         ;; restore Scheme handler, SIG_IGN or SIG_DFL.
-                        (sigaction SIGINT (car handler) (cdr handler))
+                        (sigaction SIGINT (car (handler)) (cdr (handler)))
                         ;; restore original C handler.
                         (sigaction SIGINT #f)))))
-            (lambda (k . _) (handler-thunk)))))))
+            (lambda (k . _)
+              (handler-thunk)
+              ((car (handler)) SIGINT)
+              (primitive-exit 0)))))))
 
 (define-syntax-rule (define-box-type name)
   (define-record-type name (fields treasure)))
@@ -1170,6 +1180,13 @@
       (lambda (v e)
         (or (eq? e 'ANY)
             (eq? (detect-type-name v) e)
+            (match e
+              (('or types ...)
+               (any (lambda (t)
+                      (or (equal? t v)
+                          (eq? t (detect-type-name v))))
+                    types))
+              (else #f))
             (throw 'artanis-err 500 check-function-types
                    "`Return value ~a(~a) is expected to be type `~a'"
                    v (detect-type-name v) e)))
@@ -1344,21 +1361,22 @@
 ;; ENHANCE: use colored output
 (define* (artanis-log blame-who? status mime #:key (port (current-error-port))
                       (request #f))
-  (case blame-who?
-    ((client)
-     (when (not request)
-       (error "artanis-log: Fatal bug! Request shouldn't be #f here!~%"))
-     (let* ((uri (request-uri request))
-            (path (uri-path uri))
-            (qstr (uri-query uri))
-            (method (request-method request)))
-       (format port "[Remote] ~a @ ~a~%" (remote-info request) (local-time-stamp))
-       (format port "[Request] method: ~a, path: ~a, query: ~a~%" method path qstr)
-       (format port "[Response] status: ~a, MIME: ~a~%~%" status mime)))
-    ((server)
-     (format port "[Server] ~a @ ~a~%" (get-conf '(host addr)) (local-time-stamp))
-     (format port "[Response] status: ~a, MIME: ~a~%~%" status mime))
-    (else (error "artanis-log: Fatal BUG here!"))))
+  (monitor
+   (case blame-who?
+     ((client)
+      (when (not request)
+        (error "artanis-log: Fatal bug! Request shouldn't be #f here!~%"))
+      (let* ((uri (request-uri request))
+             (path (uri-path uri))
+             (qstr (uri-query uri))
+             (method (request-method request)))
+        (format port "[Remote] ~a @ ~a~%" (remote-info request) (local-time-stamp))
+        (format port "[Request] method: ~a, path: ~a, query: ~a~%" method path qstr)
+        (format port "[Response] status: ~a, MIME: ~a~%~%" status mime)))
+     ((server)
+      (format port "[Server] ~a @ ~a~%" (get-conf '(host addr)) (local-time-stamp))
+      (format port "[Response] status: ~a, MIME: ~a~%~%" status mime))
+     (else (error "artanis-log: Fatal BUG here!")))))
 
 (define *guile-compatible-server-core*
   '(guile fibers))
@@ -1610,3 +1628,42 @@
     (if (string-null? p)
         (format #f "~a/cache/index.html" (current-tmp))
         (format #f "~a/cache/~a.html" (current-tmp) (-> path)))))
+
+(define-syntax-rule (sxml-query pattern alst)
+  (let/ec return
+    (fold
+     (lambda (x p)
+       (or (assoc-ref p x)
+           (return #f)))
+     alst
+     (string-split pattern #\/))))
+
+(define (sxml-replace! alst key val)
+  (define (list-deep-copy lst)
+    (map (lambda (x) (if (list? x) (list-deep-copy x) x)) lst))
+  (let ((new-lst (list-deep-copy alst))
+        (v (assoc-ref alst key)))
+    (cond
+     ((null? v)
+      (assoc-set! new-lst key val))
+     (else
+      (assoc-set! new-lst key (cons val v))))))
+
+(define *http-lang-tag-re* (string->irregex "([a-z]+)-([A-Z])+"))
+(define (http-lang-tag->gnu-locale lang)
+  (cond
+   ((or (not lang) (string-null? lang)) "")
+   (else
+    ;; replace - with _
+    (if (irregex-match *http-lang-tag-re* lang)
+        (irregex-replace/all "-" lang "_")
+        (throw 'artanis-err 500 http-lang-tag->gnu-locale
+               "Invalid HTTP lang tag format: ~a" lang)))))
+
+(define *gnu-locale-re* (string->irregex "([a-z]+)_([A-Z])+"))
+(define (gnu-locale->http-lang-tag lang)
+  ;; replace _ with -
+  (if (irregex-match *gnu-locale-re* lang)
+      (irregex-replace/all "_" lang "-")
+      (throw 'artanis-err 500 gnu-locale->http-lang-tag
+             "Invalid GNU locale format: ~a" lang)))
